import os
import argparse
from urllib.request import urlopen
from zipfile import ZipFile
import requests
import sys
# os.system('pip install argparse')

PERCEPTIMATIC_URLS = {
    'cogsci-2019': {'wav_source': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/sound_data/wavs_source.zip',
                    'wav_extracted': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/sound_data/wavs_extracted.zip',
                    'wav_triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/sound_data/wavs_triplets.zip',  # sound files
                    'human_data': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/annotation_data/cogsci-2019_human_experimental_data.csv',
                    'stimuli': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/annotation_data/cogsci-2019_stimuli.csv',
                    'triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/cogsci-2019/annotation_data/cogsci-2019_triplets.csv'},  # annotation files

    'zerospeech': {'wav_source': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/sound_data/wavs_source.zip',
                   'wav_extracted': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/sound_data/wavs_extracted.zip',
                   'wav_triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/sound_data/wavs_triplets.zip',
                   'human_data': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/annotation_data/zerospeech_human_experimental_data.csv',
                   'stimuli': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/annotation_data/zerospeech_stimuli.csv',
                   'triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/zerospeech/annotation_data/zerospeech_triplets.csv'},

    'pilot-july-2018': {'wav_source': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/sound_data/wavs_source.zip',
                        'wav_extracted': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/sound_data/wavs_extracted.zip',
                        'wav_triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/sound_data/wavs_triplets.zip',
                        'human_data': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/annotation_data/pilot-july-2018_human_experimental_data.csv',
                        'stimuli': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/annotation_data/pilot-july-2018_stimuli.csv',
                        'triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-july-2018/annotation_data/pilot-july-2018_triplets.csv'},

    'pilot-aug-2018': {'wav_source': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/sound_data/wavs_source.zip',
                       'wav_extracted': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/sound_data/wavs_extracted.zip',
                       'wav_triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/sound_data/wavs_triplets.zip',
                       'human_data': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/annotation_data/pilot-aug-2018_human_experimental_data.csv',
                       'stimuli': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/annotation_data/pilot-aug-2018_stimuli.csv',
                       'triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/pilot-aug-2018/annotation_data/pilot-aug-2018_triplets.csv'},

    'WorldVowels': {
        'wav_source': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/sound_data/wavs_source.zip',
        'wav_extracted': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/sound_data/wavs_extracted.zip',
        'wav_triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/sound_data/wavs_triplets.zip',
        'human_data': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/annotation_data/WorldVowels_human_experimental_data.csv',
        'stimuli': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/annotation_data/WorldVowels_stimuli.csv',
        'triplets': 'https://docs.cognitive-ml.fr/perceptimatic/data/WorldVowels/annotation_data/WorldVowels_triplets.csv'}
}


def download_sound_data_from_subset(subset_name, subset_folder_dir):
    """

    :param subset_name: (str) name of the subset
    (either 'Cogsci-2019', 'zerospeech', 'pilot-july-2018' or 'pilot-aug-2018')
    :param subset_folder_dir: (str), path to dir to save results
    :return: Downloads perceptimatic subsets and stores sound data
    """
    os.makedirs(subset_folder_dir, exist_ok=True)
    wav_dir = os.path.join(subset_folder_dir, subset_name, 'sound_data/')

    # Unzip desired files
    zip_url = PERCEPTIMATIC_URLS[subset_name]

    # Sound data
    zipresp_source = urlopen(zip_url['wav_source'])
    tempzip_source = open("/tmp/tempfile_source.zip", "wb")
    zipresp_ext = urlopen(zip_url['wav_extracted'])
    tempzip_ext = open("/tmp/tempfile_ext.zip", "wb")

    zipresp_triplets = urlopen(zip_url['wav_triplets'])
    tempzip_triplets = open("/tmp/tempfile_triplets.zip", "wb")

    # Write the contents of the downloaded file into the new file
    tempzip_source.write(zipresp_source.read())
    tempzip_ext.write(zipresp_ext.read())
    tempzip_triplets.write(zipresp_triplets.read())

    # Close the newly-created file
    tempzip_source.close()
    tempzip_ext.close()
    tempzip_triplets.close()

    # Re-open the newly-created file with ZipFile()
    zf_source = ZipFile("/tmp/tempfile_source.zip")
    zf_ext = ZipFile("/tmp/tempfile_ext.zip")
    zf_triplets = ZipFile("/tmp/tempfile_triplets.zip")

    # Extract its contents into subset_folder_dir

    # note that extractall will automatically create the path
    zf_source.extractall(path=wav_dir)
    zf_ext.extractall(path=wav_dir)
    zf_triplets.extractall(path=wav_dir)

    # close the ZipFile instance
    zf_source.close()
    zf_ext.close()
    zf_triplets.close()


def download_annotation_data_from_subset(subset_name, subset_folder_dir):
    """

    :param subset_name: (str) name of the subset
    (either 'Cogsci-2019', 'zerospeech', 'pilot-july-2018' or 'pilot-aug-2018')
    :param subset_folder_dir: (str), path to dir to save results
    :return: Downloads perceptimatic subsets and stores annotation data
    """
    annot_output_dir = os.path.join(subset_folder_dir, subset_name, 'annotation_data')
    os.makedirs(annot_output_dir, exist_ok=True)
    annot_url = PERCEPTIMATIC_URLS[subset_name]

    # Extract the csv names from the URLs
    human_res_filename = annot_url['human_data'].strip().split('/')[-1]
    stimuli_filename = annot_url['stimuli'].strip().split('/')[-1]
    triplets_filename = annot_url['triplets'].strip().split('/')[-1]

    # Extract URLs contents
    human_res_url = requests.get(annot_url['human_data'])
    stimuli_url = requests.get(annot_url['stimuli'])
    triplets_url = requests.get(annot_url['triplets'])
    human_res_url_content = human_res_url.content
    stimuli_url_content = stimuli_url.content
    triplets_url_content = triplets_url.content

    # Write url contents in csv files
    human_res_csv_file = open(os.path.join(annot_output_dir, human_res_filename), 'wb')
    stimuli_csv_file = open(os.path.join(annot_output_dir, stimuli_filename), 'wb')
    triplets_csv_file = open(os.path.join(annot_output_dir, triplets_filename), 'wb')
    human_res_csv_file.write(human_res_url_content)
    stimuli_csv_file.write(stimuli_url_content)
    triplets_csv_file.write(triplets_url_content)

    human_res_csv_file.close()
    stimuli_csv_file.close()
    triplets_csv_file.close()


def download_perceptimatic():
    parser = argparse.ArgumentParser(description='Download Perceptimatic Dataset')
    parser.add_argument("--data_folder_dir", default='Perceptimatic/', type=str, help='Directory to store the dataset')
    parser.add_argument("--data_to_load", default=["cogsci-2019", "zerospeech", "pilot-july-2018", "pilot-aug-2018", "WorldVowels"],
                        type=str, help='Name of perceptimatic subsets to download')
    parser.add_argument("--data_options", default="all",
                        type=str, help='what the user wants to download',
                        choices=['sound_only', 'annotations_only', 'all'])
    args = parser.parse_args()

    # extract perceptimatic chosen subsets
    # files_to_load = args.data_to_load.strip().split(',')

    for subset_name in args.data_to_load:
        sys.stdout.write('\r Downloading {} dataset ...'.format(subset_name))

        # Download sound data
        if args.data_options == 'sound_only' or args.data_options == 'all':
            download_sound_data_from_subset(subset_name, args.data_folder_dir)

        # Download annotation data
        if args.data_options == 'annotations_only' or args.data_options == 'all':
            download_annotation_data_from_subset(subset_name, args.data_folder_dir)


if __name__ == '__main__':
    download_perceptimatic()
    print('Script DONE')
